# -*- coding: utf-8 -*-


import telegram
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


class Bot(telegram.Bot):
    __bot__ = None

    _DEFAULT_PARSE_MODE = telegram.ParseMode.MARKDOWN

    @staticmethod
    def get_instance():
        if Bot.__bot__ is None:
            Bot.__bot__ = Bot._config()
            return Bot.__bot__
        else:
            return Bot.__bot__

    @staticmethod
    def _config():
        token = getattr(settings, 'LAMAGRAM_TOKEN', None)
        if token is None:
            raise ImproperlyConfigured('The LAMAGRAM_TOKEN setting must not be empty.')
        return Bot(token=token)

    def __init__(self, token):
        super(Bot, self).__init__(token=token)

    def send_message(self, chat_id, text, parse_mode=None, **kwargs):
        return super(Bot, self).send_message(
            chat_id=chat_id,
            text=text,
            parse_mode=parse_mode,
            **kwargs,
        )

    def send_code(self, chat_id, text):
        text = F"`{text}`"
        parse_mode = self._DEFAULT_PARSE_MODE
        return self.send_message(chat_id, text, parse_mode)

    def send_language_code(self, chat_id, text):
        text = F"```python\n{text}```"
        parse_mode = self._DEFAULT_PARSE_MODE
        return self.send_message(chat_id, text, parse_mode)



from rest_framework.viewsets import ModelViewSet as DrfModelViewSet
from rest_framework.response import Response
from rest_framework import status


class ModelViewSet(DrfModelViewSet):

    def retrieve(self, request, *args, **kwargs):
        instance = self.get_object()
        serializer = self.get_serializer(instance)
        response = {
            'data': serializer.data
        }
        return Response(response, status=status.HTTP_200_OK)



from ..command import Command
from .utils.functions import config_scope, Level, validate_status
from sentry_sdk import capture_event


class SentryCommand(Command):
    def __init__(self, response, status_code):
        super(SentryCommand, self).__init__()
        self._response = response
        self._status_code = status_code

    def execute(self):
        response = self._response
        status = response.get('status')
        metadata = response.get('metadata')
        status_code = self._status_code
        if validate_status(status_code):
            if status == 'failed':
                config_scope(metadata, status_code, Level.ERROR)
                sentry_exception_code = capture_event({
                    'error': response.get('error')
                })
            elif status == 'success':
                config_scope(metadata, status_code, Level.INFO)
                sentry_exception_code = capture_event(
                    {
                        'data': response.get('data'),
                        'message': 'request was successfully submitted'
                    }
                )
            else:
                config_scope(metadata, status_code, Level.ERROR)
                sentry_exception_code = capture_event(
                    {
                        'response': response,
                        'message': F'status is invalid ({status})'
                    }
                )
            return {'sentry_code': sentry_exception_code}
        else:
            return {}



def get_metadata(locale, user=None, request_data=None):
    if user is None:
        user = {}
    return {
        "locale": locale,
        "user": {
            "id": user.get("id", None),
            "username": user.get("username", None)
        },
        'request_data': request_data if request_data else {},
        "transaction": []
    }



import sentry_sdk
from django.conf import settings
from sentry_sdk.integrations.django import DjangoIntegration
from sentry_sdk.integrations.celery import CeleryIntegration


def init():
    url = getattr(settings, 'SENTRY_URL', None)
    key = getattr(settings, 'SENTRY_KEY', None)
    organization = getattr(settings, 'SENTRY_ORGANIZATION', None)
    project = getattr(settings, 'SENTRY_PROJECT', None)
    if url:
        sentry_sdk.init(
            dsn=url,
            integrations=[DjangoIntegration(), CeleryIntegration()],
            send_default_pii=True
        )
    elif key and organization and project:
        sentry_sdk.init(
            dsn=F"https://{key}@{organization}.ingest.sentry.io/{project}",
            integrations=[DjangoIntegration(), CeleryIntegration()],
            send_default_pii=True
        )
    else:
        raise Exception('sentry config is not provided')



from .serializer import ReportSerializer
from .models import Report
from tamarin.rest.viewset import ReadOnlyModelViewSet
from rest_framework_api_key.permissions import HasAPIKey


class ReportApi(ReadOnlyModelViewSet):
    queryset = Report.objects.all()
    serializer_class = ReportSerializer
    permission_classes = [HasAPIKey, ]



from rest_framework import permissions


class IsStaff(permissions.BasePermission):

    def has_permission(self, request, view):
        is_not_anonymous = str(request.user) != 'AnonymousUser'
        return is_not_anonymous and request.user.get('is_active') and request.user.get('is_staff')


class IsUserStaff(permissions.BasePermission):

    def has_permission(self, request, view):
        is_not_anonymous = str(request.user) != 'AnonymousUser'
        is_active = request.user.is_active
        is_staff = request.user.is_staff
        return is_not_anonymous and is_active and is_staff



from .pool import Pool


class Todo(object):
    def __init__(self, commands):
        self._commands = commands

    def execute(self, command):
        return command.execute()

    def process(self):
        self._in_progress()
        self._done()
        return getattr(self, 'response')

    def _in_progress(self):
        command_count = len(self._commands)
        processes_count = command_count or 1
        with Pool(processes=processes_count) as pool:
            response = pool.map(self.execute, self._commands)
        setattr(self, 'response', response)

    def _done(self):
        response = getattr(self, 'response')
        if not response:
            raise RuntimeError('work must progress before calling done')



from ..todo import LogToDo
import elasticapm

def log(response, status_code, index: str, doc, args=None, params=None, headers=None):
    metadata = response.get('metadata')
    status = response.get('status')
    elasticapm.set_context(
        response
    )
    elasticapm.label(
        apm_transaction_id=elasticapm.get_transaction_id(),
        index=index,
        status=response.get('status')
    )
    todo = LogToDo(response, status_code, index, doc, args, headers, params)
    results = todo.process()
    transaction = {
        'status': status,
        'index': index,
        'apm_transaction_id': elasticapm.get_transaction_id(),
    }
    for result in results:
        transaction.update(result)

    metadata.update({
        'transaction': transaction
    })
    return response, status_code



import time


def retry_with_backoff(func, retries=5, backoff_in_seconds=1, exception_type=Exception, **kwargs):
    counter = 0
    while True:
        try:
            return func(**kwargs)
        except exception_type as e:
            if counter == retries:
                raise exception_type(e.message)
            else:
                sleep = (backoff_in_seconds * 2 ** counter)
                time.sleep(sleep)
                counter += 1



from logging import StreamHandler, critical
from django.conf import settings
from logging import LogRecord
import requests


class LamagramHandler(StreamHandler):

    def __init__(self, url=None, token=None):
        super(LamagramHandler, self).__init__()

    def emit(self, record: LogRecord):
        message = self.format(record)
        try:
            requests.post(
                getattr(settings, 'TITI_URL', ''),
                json=message,
                headers={
                    "Authorization": getattr(settings, 'TITI_TOKEN', '')
                }
            )
        except requests.ConnectionError:
            critical('connect to titi failed')
        except Exception:
            critical('error has been occured (send bonobo-exception)')

    def format(self, record: LogRecord):
        message = F"{record.name} `{record.module}` \n" \
                  F"`{record.pathname}` {record.lineno} \n" \
                  F"`{record.getMessage()}`"
        return {
            'message': message,
            'code': 'bonobo-exception',
        }



from rest_framework_simplejwt.views import TokenObtainPairView, TokenRefreshView as TamarinRefreshView
from rest_framework_simplejwt.serializers import TokenObtainPairSerializer


class TamarinTokenSerializer(TokenObtainPairSerializer):

    @classmethod
    def get_token(cls, user):
        token = super().get_token(user)

        token['pk'] = user.pk
        token['first-name'] = user.first_name
        token['last-name'] = user.last_name
        token['full-name'] = user.get_full_name()
        token['email'] = user.email
        token['username'] = user.username
        token['last-login'] = str(user.last_login)
        token['firebase-token'] = getattr(user, 'firebase_token', '')
        token['is_active'] = user.is_active
        token['is_staff'] = user.is_staff
        token['groups'] = [group.name for group in user.groups.all()]
        token['permissions'] = [permission.split('.')[1] for permission in user.get_all_permissions()]
        return token


class TamarinTokenObtainPairView(TokenObtainPairView):
    serializer_class = TamarinTokenSerializer



from rest_framework.permissions import BasePermission
from django.contrib.auth import get_user_model
from django.db.models import Q


class HasPermission(BasePermission):

    def has_permission(self, request, view):
        permissions = getattr(self, 'filter')
        if permissions:
            if type(permissions) == list:
                user_permissions = request.user.get('permissions')
                for permission in permissions:
                    if not (permission in user_permissions):
                        return False
                return True
            elif type(permissions) == str:
                return bool(permissions in request.user.get('permissions'))
            else:
                raise AttributeError('permissions type is not list or string')
        else:
            raise AttributeError('permissions does not set')


class HasUserPermission(BasePermission):

    def has_permission(self, request, view):
        User = get_user_model()
        permissions = getattr(self, 'filter')
        if permissions:
            if type(permissions) == list:
                queryset = User.objects.filter(
                    Q(pk=request.user.pk),
                    Q(user_permissions__codename__in=permissions) | Q(groups__permissions__codename__in=permissions),
                )
                return queryset.exists()
            elif type(permissions) == str:
                queryset = User.objects.filter(
                    Q(pk=request.user.pk),
                    Q(user_permissions__codename=permissions) | Q(groups__permissions__codename=permissions),
                )
                return queryset.exists()
            else:
                raise AttributeError('permissions type is not list or string')
        else:
            raise AttributeError('permissions does not set')



from rest_framework.pagination import PageNumberPagination
from rest_framework.response import Response


class Pagination(PageNumberPagination):
    page_size = 5
    page_query_param = 'page'
    page_size_query_param = 'page_size'
    max_page_size = 1000

    def paginate_queryset(self, queryset, request, view=None):
        setattr(self, 'count', self.get_count(queryset))
        return super(Pagination, self).paginate_queryset(queryset, request, view)

    def get_count(self, queryset):
        """
        Determine an object count, supporting either querysets or regular lists.
        """
        try:
            return queryset.count()
        except (AttributeError, TypeError):
            return len(queryset)

    def get_paginated_response(self, data):
        return Response({
            'current': self.page.number,
            'count': getattr(self, 'count', '-'),
            **data
        })



from tamarin.command import Command
from .models import Report


class ReportCommand(Command):
    def __init__(self, index, status_code, json=None, text=None):
        super(ReportCommand, self).__init__()
        self.index = index
        self.status = status_code
        self.json = json
        self.text = text

    def execute(self):
        try:
            report = Report.objects.create(
                index=self.index,
                status=self.status,
                data=self.json,
                text=self.text,
            )
        except TypeError:
            report = Report.objects.create(
                index=self.index,
                status=self.status,
                text=self.json,
            )
        return {
            'report': report.id
        }



from .config import HOST, PORT, USE_SSL, PROTOCOL, USER, SECRET
from elasticsearch import Elasticsearch


class ElasticSearchClient:
    client: Elasticsearch = None

    @classmethod
    def get_client(cls):
        if not ElasticSearchClient.client:
            hosts = [
                {
                    "host": HOST,
                    "port": PORT
                }
            ]
            ElasticSearchClient.client = Elasticsearch(
                hosts=hosts,
                use_ssl=USE_SSL,
                scheme=PROTOCOL,
                http_auth=(USER, SECRET) if USER else None
            )
        return ElasticSearchClient.client

    @classmethod
    def info(cls):
        if ElasticSearchClient.client:
            return ElasticSearchClient.client.info()
        return {}

    @classmethod
    def health(cls):
        if ElasticSearchClient.client:
            return ElasticSearchClient.client.cluster.health()
        return {}



from typing import Dict

from firebase_admin import firestore, initialize_app, credentials, messaging
from ...connections.client import Client
from django.conf import settings
import datetime
import os

_DEFAULT_APP_NAME = 'MESSAGING_DEFAULT'


class MessagingClient(Client):
    __app__ = None

    __messaging_client__ = None

    __credential__ = credentials.Certificate(
        os.environ.get(
            'GOOGLE_APPLICATION_CREDENTIALS',
            getattr(
                settings,
                'GOOGLE_APPLICATION_CREDENTIALS'
            )
        )
    )

    def _get_access_token(self):
        credential = self.__credential__
        access_token_info = credential.get_access_token()
        return access_token_info.access_token

    @staticmethod
    def create_notification(title, body) -> messaging.Notification:
        notification = messaging.Notification(
            title=title,
            body=body,
        )
        return notification

    @staticmethod
    def create_webpush(title, body, icon, actions=None) -> messaging.WebpushConfig:
        webpush = messaging.WebpushConfig(
            notification=messaging.WebpushNotification(
                title=title,
                body=body,
                icon=icon,
                actions=actions
            )
        )
        return webpush

    @staticmethod
    def create_apns_config(priority, title, body, badge=30):
        apns_config = messaging.APNSConfig(
            headers={'apns-priority': str(priority)},
            payload=messaging.APNSPayload(
                aps=messaging.Aps(
                    alert=messaging.ApsAlert(
                        title=title,
                        body=body,
                    ),
                    badge=badge,
                ),
            ),
        )
        return apns_config

    @staticmethod
    def create_android_config(title, body, icon=None, color=None, priority='normal', action=None):
        """

        :param title:
        :param body:
        :param icon:
        :param color: #rrggbb format
        :param priority: default 'normal'
        :param action:
        :return:
        """
        android_config = messaging.AndroidConfig(
            ttl=datetime.timedelta(seconds=3600),
            priority=priority,
            notification=messaging.AndroidNotification(
                title=title,
                body=body,
                icon=icon,
                color=color,
                click_action=action
            ),
        )
        return android_config

    @staticmethod
    def create_message(token=None, token_list=None, topic=None, condition=None, data: dict = None,
                       notification: messaging.Notification = None, webpush: messaging.WebpushConfig = None,
                       android: messaging.AndroidConfig = None, apns: messaging.APNSConfig = None):
        if token:
            message = messaging.Message(
                data=data,
                notification=notification,
                webpush=webpush,
                android=android,
                apns=apns,
                token=token,
            )
        elif token_list:
            message = messaging.MulticastMessage(
                data=data,
                notification=notification,
                webpush=webpush,
                android=android,
                apns=apns,
                tokens=token_list,
            )
        elif topic:
            message = messaging.Message(
                data=data,
                notification=notification,
                webpush=webpush,
                android=android,
                apns=apns,
                topic=topic,
            )
        elif condition:
            message = messaging.Message(
                data=data,
                notification=notification,
                webpush=webpush,
                android=android,
                apns=apns,
                condition=condition,
            )
        else:
            raise AttributeError("One of token, token_list, topic or condition should'nt None")

    @staticmethod
    def get_instance():
        return MessagingClient.__messaging_client__ or MessagingClient()

    def __init__(self):
        if MessagingClient.__messaging_client__:
            raise Exception("Messaging is singleton, just one instance")
        else:
            super(MessagingClient, self).__init__()
            credential = MessagingClient.__credential__
            options = getattr(settings, 'MESSAGING_APP_OPTIONS', None)
            name = getattr(settings, 'MESSAGING_APP_NAME', _DEFAULT_APP_NAME)
            app = initialize_app(credential=credential, options=options, name=name)
            MessagingClient.__app__ = app
            MessagingClient.__messaging_client__ = self

    def send(self, token=None, token_list=None, topic=None, condition=None, data: dict = None,
             notification: messaging.Notification = None, webpush: messaging.WebpushConfig = None,
             android: messaging.AndroidConfig = None, apns: messaging.APNSConfig = None, dry_run: bool = False):
        if token:
            return self._send_device(token, data, notification, webpush, android, apns, dry_run)
        elif token_list:
            return self.send_multicast(token_list, data, notification, webpush, android, apns, dry_run)
        elif topic:
            return self._send_to_topic(topic, data, notification, webpush, android, apns, dry_run)
        elif condition:
            return self.send_to_condition(condition, data, notification, webpush, android, apns, dry_run)
        else:
            raise AttributeError("One of token, token_list, topic or condition should'nt None")

    def _send_device(self, token, data: dict = None, notification: messaging.Notification = None,
                     webpush: messaging.WebpushConfig = None, android: messaging.AndroidConfig = None,
                     apns: messaging.APNSConfig = None, dry_run: bool = False):
        app = self.__app__
        message = messaging.Message(
            data=data,
            notification=notification,
            webpush=webpush,
            android=android,
            apns=apns,
            token=token,
        )
        response = messaging.send(
            message=message,
            dry_run=dry_run,
            app=app
        )
        return response

    def _send_to_topic(self, topic, data: dict = None, notification: messaging.Notification = None,
                       webpush: messaging.WebpushConfig = None, android: messaging.AndroidConfig = None,
                       apns: messaging.APNSConfig = None, dry_run: bool = False):
        app = self.__app__
        message = messaging.Message(
            data=data,
            notification=notification,
            webpush=webpush,
            android=android,
            apns=apns,
            topic=topic,
        )
        response = messaging.send(
            message=message,
            dry_run=dry_run,
            app=app
        )
        return response

    def send_to_condition(self, condition, data: dict = None, notification: messaging.Notification = None,
                          webpush: messaging.WebpushConfig = None, android: messaging.AndroidConfig = None,
                          apns: messaging.APNSConfig = None, dry_run: bool = False):
        """
        :param android:
        :param notification:
        :param data:
        :param dry_run:
        :param apns:
        :param webpush:
        :param condition: like "'stock' in topics || 'industry-tech' in topics"
        """
        app = self.__app__
        message = messaging.Message(
            data=data,
            notification=notification,
            webpush=webpush,
            android=android,
            apns=apns,
            condition=condition,
        )
        response = messaging.send(
            message=message,
            dry_run=dry_run,
            app=app
        )
        return response

    def subscribe_to_topic(self, topic: str, tokens: list):
        app = self.__app__
        response = messaging.subscribe_to_topic(tokens, topic, app)
        return response

    def unsubscribe_from_topic(self, topic: str, tokens: list):
        app = self.__app__
        response = messaging.unsubscribe_from_topic(tokens, topic, app)
        return response

    def send_all(self, messages, dry_run):
        """
        :param dry_run:
        :param messages: Create a list containing up to 500 messages
        :return:
        """
        app = self.__app__
        response = messaging.send_all(messages, dry_run, app)
        return response

    def send_multicast(self, tokens: list, data: dict = None, notification: messaging.Notification = None,
                       webpush: messaging.WebpushConfig = None, android: messaging.AndroidConfig = None,
                       apns: messaging.APNSConfig = None, dry_run: bool = False):
        """

        :param tokens: # Create a list containing up to 500 registration tokens.
        :param data:
        :param notification:
        :param webpush:
        :param android:
        :param apns:
        :param dry_run:
        :return:
        """
        app = self.__app__
        message = messaging.MulticastMessage(
            data=data,
            notification=notification,
            webpush=webpush,
            android=android,
            apns=apns,
            tokens=tokens,
        )
        response = messaging.send_multicast(message, dry_run, app)
        return response



# Generated by Django 3.1.4 on 2021-01-07 13:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('logs', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Report',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_created=True, auto_now_add=True)),
                ('index', models.CharField(max_length=1000)),
                ('status', models.PositiveSmallIntegerField()),
                ('data', models.JSONField(blank=True, null=True)),
                ('text', models.TextField(blank=True, null=True)),
            ],
            options={
                'ordering': ('-created_at', 'index'),
            },
        ),
    ]



import unittest
from tamarin.command import Command
from tamarin.multiprocessing import Todo


class LightTurnOn(Command):
    def execute(self):
        print('light turn on')
        return 'light turn on'


class WakeUp(Command):
    def execute(self):
        print('wake up')
        return 'wake_up'


class MultiProcessingTest(unittest.TestCase):
    first_command = LightTurnOn(None)
    second_command = WakeUp(None)

    def setUp(self) -> None:
        pass

    def test_todo(self):
        commands = [self.first_command, self.second_command]
        todo = Todo(commands=commands)
        response = todo.process()



from django.db.utils import ConnectionRouter
from django.db.models import Model


class LogRouter(ConnectionRouter):
    route_app_labels = {'logs'}

    def db_for_read(self, model: Model, **hints):
        """ reading Log from log """
        if model._meta.app_label in self.route_app_labels:
            return 'log'
        return None

    def db_for_write(self, model: Model, **hints):
        """ writing Log to log """
        if model._meta.app_label in self.route_app_labels:
            return 'log'
        return None

    def allow_relation(self, obj1: Model, obj2: Model, **hints):
        """
        Allow relations if a model in the log apps is
        involved.
        """
        if (
                obj1._meta.app_label in self.route_app_labels or
                obj2._meta.app_label in self.route_app_labels
        ):
            return True
        return None

    def allow_migrate(self, db, app_label, model_name=None, **hints):
        """
        Make sure the log apps only appear in the
        'log' database.
        """
        if app_label in self.route_app_labels:
            return db == 'log'
        elif db == 'log':
            return False
        else:
            pass



from django.conf import settings
from tamarin import status

SENTRY_ALLOWED_ALL = getattr(settings, 'SENTRY_ALLOWED_ALL', False)
SENTRY_ALLOWED_STATUS = getattr(settings, 'SENTRY_ALLOWED_STATUS', [])


def validate_status(status_code):
    if SENTRY_ALLOWED_ALL:
        return True
    allowed_condition = [getattr(status, F'is_{method}', None) for method in SENTRY_ALLOWED_STATUS]
    for condition in allowed_condition:
        if condition and condition(status_code):
            return True
    if status_code in SENTRY_ALLOWED_STATUS:
        return True
    return False



from functools import wraps
from ..functions import log as log_provider


def log(index: str, doc, params=None, headers=None):
    def decorator(func):
        @wraps(func)
        def inner(*args, **kwargs):
            response, status_code = func(*args, **kwargs)
            response, status_code = log_provider(
                response=response,
                status_code=status_code,
                index=index,
                doc=doc,
                params=params,
                headers=headers
            )
            return response, status_code
        return inner

    return decorator



from ..multiprocessing import Todo
from ..sentry.commands import SentryCommand
from ..elasticsearch.commands import ElasticSearchCommand
from .commands import ReportCommand
from ..lamagram.commands import LamagramCommand


class LogToDo(Todo):
    def __init__(self, response, status_code, index, doc, args, headers, params):
        sentry_cmd = SentryCommand(response, status_code)
        elastic_cmd = ElasticSearchCommand(response, status_code, index, doc, args, headers, params)
        report_cmd = ReportCommand(index, status_code, response)
        commands = [sentry_cmd, elastic_cmd, report_cmd]
        super(LogToDo, self).__init__(commands)


class LamagramToDo(Todo):
    def __init__(self, chat_id, text, status_code):
        lamagram_cmd = LamagramCommand(chat_id, text, status_code)
        commands = [lamagram_cmd]
        super(LamagramToDo, self).__init__(commands)



from rest_framework.permissions import BasePermission


class IsAuthenticated(BasePermission):

    def has_permission(self, request, view):
        is_not_anonymous = str(request.user) != 'AnonymousUser'
        return is_not_anonymous and request.user.get('is_active')


class IsUserAuthenticated(BasePermission):
    def has_permission(self, request, view):
        is_not_anonymous = str(request.user) != 'AnonymousUser'
        is_active = request.user.is_active
        return is_not_anonymous and is_active



from tamarin.command import Command
from .bot import Bot
from sentry_sdk import capture_exception
from .utils.functions import validate_status


class LamagramCommand(Command):
    def __init__(self, chat_id, text, status_code):
        super(LamagramCommand, self).__init__()
        self.chat_id = chat_id
        self.text = text
        self.status = status_code

    def execute(self):
        if validate_status(status_code=self.status):
            try:
                bot = Bot.get_instance()
                bot.send_code(self.chat_id, self.text)
            except Exception as e:
                capture_exception(e)



from django.contrib import admin
from .models import Log, Report

@admin.register(Log)
class LogAdmin(admin.ModelAdmin):
    using = 'log'

    def get_queryset(self, request):
        return super(LogAdmin, self).get_queryset(request).using(self.using)


@admin.register(Report)
class ReportAdmin(admin.ModelAdmin):
    using = 'log'

    list_display = ['id', 'index', 'status', 'created_at']
    list_filter = ['created_at', 'status', 'index']
    search_fields = ['index']

    def get_queryset(self, request):
        return super(ReportAdmin, self).get_queryset(request).using(self.using)



import inspect


def var2str(data):
    data_type = type(data)
    if inspect.getmro(data_type).__contains__(dict):
        snake_data = {}
        for key in data:
            snake_data.update({
                str(key): var2str(
                    data.get(key)
                )
            })
    elif inspect.getmro(data_type).__contains__(list):
        snake_data = []
        for item in data:
            snake_data.append(var2str(item))
    elif inspect.getmro(data_type).__contains__(set):
        snake_data = set()
        for item in data:
            snake_data.add(var2str(item))
    else:
        snake_data = str(data)
    return snake_data
